/*
 * A resource offering support for creating processes.
 *
 * Copyright (C) 2023 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <systypes/fcntl.h>

#include "process_creator_resource.h"
#include "process_creator_context_resource.h"
#include "process_creator_context_object_server.h"

#include <string.h>
#include <stdio.h>



/* Support for creating processes. */

ProcessCreatorContextResource::ProcessCreatorContextResource(ProcessCreatorResource *creator)
: SimplePager(), _creator(creator)
{
}

ProcessCreatorContextResource::~ProcessCreatorContextResource()
{
}

ipc_server_default_config_type ProcessCreatorContextResource::config()
{
    return config_ProcessCreatorContextObject;
}



/* ProcessCreatorContext interface methods. */

long ProcessCreatorContextResource::start(int argc, l4_cap_idx_t *process)
{
    /* Obtain the arguments by reading from the shared memory. */

    const char *argv[argc];
    offset_t pos = 0;

    for (int i = 0; i < argc; i++)
    {
        argv[i] = get_string(pos);

        if (argv[i] != NULL)
            pos += strlen(argv[i]) + 1;

        printf("argv[%d] = %s\n", i, argv[i]);
    }

    /* Start the new process, obtaining a reference to it. */

    return _creator->start(argc, argv, process);
}

/* vim: tabstop=4 expandtab shiftwidth=4
*/
